@@ -4,30 +4,17 @@
 #
 
 config TARGET_ARCH
-	default "sh" if CONFIG_SH2 || CONFIG_SH3 || CONFIG_SH4
-	default "sh64" if CONFIG_SH5
+	string
+	default "sh"
 
-config HAVE_ELF
+config FORCE_OPTIONS_FOR_ARCH
 	bool
 	default y
+	select ARCH_ANY_ENDIAN
 
 config ARCH_CFLAGS
 	string
 
-config ARCH_LDFLAGS
-	string
-
-config LIBGCC_CFLAGS
-	string
-
-config HAVE_DOT_HIDDEN
-        bool
-	default y
-
-config UCLIBC_COMPLETELY_PIC
-        bool
-	default y
-
 choice
 	prompt "Target Processor Type"
 	default CONFIG_SH4
@@ -38,29 +25,31 @@ choice
 	  that uClibc will work at all....
 
 	  Here are the available choices:
+	  - "SH2A" Renesas SH-2A (SH7206)
 	  - "SH2" SuperH SH-2
 	  - "SH3" SuperH SH-3
 	  - "SH4" SuperH SH-4
-	  - "SH5" SuperH SH-5 101, 103
+
+config CONFIG_SH2A
+	select ARCH_HAS_NO_MMU
+	select HAVE_NO_PIC
+	bool "SH2A"
 
 config CONFIG_SH2
 	select ARCH_HAS_NO_MMU
-	select ARCH_HAS_NO_LDSO
 	bool "SH2"
 
 config CONFIG_SH3
-	select UCLIBC_HAS_MMU
+	select ARCH_HAS_MMU
 	bool "SH3"
 
 config CONFIG_SH4
-	select UCLIBC_HAS_MMU
 	select FORCE_SHAREABLE_TEXT_SEGMENTS
 	bool "SH4"
 
-config CONFIG_SH5
-	select UCLIBC_HAS_MMU
-	bool "SH5"
-
 endchoice
 
-
+config ARCH_HAS_BWD_MEMCPY
+       bool
+       default y
+       depends on CONFIG_SH4
