@@ -6,14 +6,18 @@
 #ifdef L_msgctl
 
 #ifdef __NR_msgctl
-_syscall3(int, msgctl, int, msqid, int, cmd, struct msqid_ds *, buf);
-#else
-/* Message queue control operation.  */
-int msgctl (int msqid, int cmd, struct msqid_ds *buf)
-{
-    return __ipc(IPCOP_msgctl ,msqid ,cmd ,0 ,buf);
-}
+#define __NR___libc_msgctl __NR_msgctl
+static __inline__ _syscall3(int, __libc_msgctl, int, msqid, int, cmd, struct msqid_ds *, buf);
 #endif
+/* Message queue control operation.  */
+int msgctl(int msqid, int cmd, struct msqid_ds *buf)
+{
+#ifdef __NR_msgctl
+	return __libc_msgctl(msqid, cmd | __IPC_64, buf);
+#else
+    return __syscall_ipc(IPCOP_msgctl, msqid, cmd | __IPC_64, 0, buf, 0);
+#endif
+}
 #endif
 
 
@@ -24,7 +28,7 @@ _syscall2(int, msgget, key_t, key, int, msgflg)
 /* Get messages queue.  */
 int msgget (key_t key, int msgflg)
 {
-    return __ipc(IPCOP_msgget ,key ,msgflg ,0 ,0);
+    return __syscall_ipc(IPCOP_msgget ,key ,msgflg ,0 ,0, 0);
 }
 #endif
 #endif
@@ -32,14 +36,14 @@ int msgget (key_t key, int msgflg)
 
 struct new_msg_buf{
     struct msgbuf * oldmsg;
-    long int r_msgtyp;       /* the fifth arg of __ipc */
+    long int r_msgtyp;       /* the fifth arg of __syscall_ipc */
 };
 /* Receive message from message queue.  */
 
 
 #ifdef L_msgrcv
 #ifdef __NR_msgrcv
-_syscall5(int, msgrcv, int, msqid, void *, msgp, size_t, msgsz, long int, msgtyp, int, msgflg);
+_syscall5(int, msgrcv, int, msqid, void *, msgp, size_t, msgsz, long int, msgtyp, int, msgflg)
 #else
 int msgrcv (int msqid, void *msgp, size_t msgsz,
 	long int msgtyp, int msgflg)
@@ -48,7 +52,7 @@ int msgrcv (int msqid, void *msgp, size_t msgsz,
 
     temp.r_msgtyp = msgtyp;
     temp.oldmsg = msgp;
-    return __ipc(IPCOP_msgrcv ,msqid ,msgsz ,msgflg ,&temp);
+    return __syscall_ipc(IPCOP_msgrcv ,msqid ,msgsz ,msgflg ,&temp, 0);
 }
 #endif
 #endif
@@ -57,12 +61,12 @@ int msgrcv (int msqid, void *msgp, size_t msgsz,
 
 #ifdef L_msgsnd
 #ifdef __NR_msgsnd
-_syscall4(int, msgsnd, int, msqid, const void *, msgp, size_t, msgsz, int, msgflg);
+_syscall4(int, msgsnd, int, msqid, const void *, msgp, size_t, msgsz, int, msgflg)
 #else
 /* Send message to message queue.  */
 int msgsnd (int msqid, const void *msgp, size_t msgsz, int msgflg)
 {
-    return __ipc(IPCOP_msgsnd, msqid, msgsz, msgflg, (void *)msgp);
+    return __syscall_ipc(IPCOP_msgsnd, msqid, msgsz, msgflg, (void *)msgp, 0);
 }
 #endif
 #endif
