@@ -27,7 +27,6 @@
  * SUCH DAMAGE.
  */
 
-#define _GNU_SOURCE
 #include <features.h>
 #include <ttyent.h>
 #include <stdio.h>
@@ -35,26 +34,33 @@
 #include <ctype.h>
 #include <string.h>
 #include <stdlib.h>
+#ifdef __UCLIBC_HAS_THREADS__
+#include <pthread.h>
+#endif
+
+/* Experimentally off - libc_hidden_proto(strchr) */
+/* Experimentally off - libc_hidden_proto(strcmp) */
+/* Experimentally off - libc_hidden_proto(strncmp) */
+libc_hidden_proto(__fsetlocking)
+libc_hidden_proto(rewind)
+libc_hidden_proto(fgets_unlocked)
+libc_hidden_proto(getc_unlocked)
+libc_hidden_proto(__fgetc_unlocked)
+libc_hidden_proto(fopen)
+libc_hidden_proto(fclose)
+libc_hidden_proto(abort)
+#ifdef __UCLIBC_HAS_XLOCALE__
+libc_hidden_proto(__ctype_b_loc)
+#elif defined __UCLIBC_HAS_CTYPE_TABLES__
+libc_hidden_proto(__ctype_b)
+#endif
 
 static char zapchar;
 static FILE *tf;
 static struct ttyent tty;
 
 
-struct ttyent * getttynam(const char *tty)
-{
-    register struct ttyent *t;
-
-    setttyent();
-    while ((t = getttyent()))
-		if (!strcmp(tty, t->ty_name))
-			break;
-    endttyent();
-    return (t);
-}
-
-
-/* Skip over the current field, removing quotes, and return 
+/* Skip over the current field, removing quotes, and return
  * a pointer to the next field.
  */
 #define	QUOTED	1
@@ -64,27 +70,27 @@ static char * skip(register char *p)
     register int c, q;
 
     for (q = 0, t = p; (c = *p) != '\0'; p++) {
-		if (c == '"') {
-			q ^= QUOTED;	/* obscure, but nice */
-			continue;
-		}
-		if (q == QUOTED && *p == '\\' && *(p+1) == '"')
-			p++;
-		*t++ = *p;
-		if (q == QUOTED)
-			continue;
-		if (c == '#') {
-			zapchar = c;
-			*p = 0;
-			break;
-		}
-		if (c == '\t' || c == ' ' || c == '\n') {
-			zapchar = c;
-			*p++ = 0;
-			while ((c = *p) == '\t' || c == ' ' || c == '\n')
-				p++;
-			break;
-		}
+	if (c == '"') {
+	    q ^= QUOTED;	/* obscure, but nice */
+	    continue;
+	}
+	if (q == QUOTED && *p == '\\' && *(p+1) == '"')
+	    p++;
+	*t++ = *p;
+	if (q == QUOTED)
+	    continue;
+	if (c == '#') {
+	    zapchar = c;
+	    *p = 0;
+	    break;
+	}
+	if (c == '\t' || c == ' ' || c == '\n') {
+	    zapchar = c;
+	    *p++ = 0;
+	    while ((c = *p) == '\t' || c == ' ' || c == '\n')
+		p++;
+	    break;
+	}
     }
     *--t = '\0';
     return (p);
@@ -93,9 +99,28 @@ static char * skip(register char *p)
 static char * value(register char *p)
 {
 
-    return ((p = index(p, '=')) ? ++p : NULL);
+    return ((p = strchr(p, '=')) ? ++p : NULL);
 }
 
+libc_hidden_proto(setttyent)
+int setttyent(void)
+{
+
+    if (tf) {
+	rewind(tf);
+	return (1);
+    } else if ((tf = fopen(_PATH_TTYS, "r"))) {
+	/* We do the locking ourselves.  */
+#ifdef __UCLIBC_HAS_THREADS__
+	__fsetlocking (tf, FSETLOCKING_BYCALLER);
+#endif
+	return (1);
+    }
+    return (0);
+}
+libc_hidden_def(setttyent)
+
+libc_hidden_proto(getttyent)
 struct ttyent * getttyent(void)
 {
     register int c;
@@ -104,43 +129,43 @@ struct ttyent * getttyent(void)
     struct ttyent *retval = NULL;
 
     if (!tf && !setttyent())
-		return (NULL);
+	return (NULL);
 
     if (!line) {
-		line = malloc(BUFSIZ);
+            line = malloc(BUFSIZ);
 		if (!line)
 		    abort();
     }
 
-    __STDIO_ALWAYS_THREADLOCK(tf);
+	__STDIO_ALWAYS_THREADLOCK(tf);
 
     for (;;) {
-		if (!fgets_unlocked(p = line, BUFSIZ, tf)) {
-			goto DONE;
-		}
-		/* skip lines that are too big */
-		if (!index(p, '\n')) {
-			while ((c = getc_unlocked(tf)) != '\n' && c != EOF)
-				;
-			continue;
-		}
-		while (isspace(*p))
-			++p;
-		if (*p && *p != '#')
-			break;
+	if (!fgets_unlocked(p = line, BUFSIZ, tf)) {
+	    goto DONE;
+	}
+	/* skip lines that are too big */
+	if (!strchr(p, '\n')) {
+	    while ((c = getc_unlocked(tf)) != '\n' && c != EOF)
+		;
+	    continue;
+	}
+	while (isspace(*p))
+	    ++p;
+	if (*p && *p != '#')
+	    break;
     }
 
     zapchar = 0;
     tty.ty_name = p;
     p = skip(p);
     if (!*(tty.ty_getty = p))
-		tty.ty_getty = tty.ty_type = NULL;
+	tty.ty_getty = tty.ty_type = NULL;
     else {
-		p = skip(p);
-		if (!*(tty.ty_type = p))
-			tty.ty_type = NULL;
-		else
-			p = skip(p);
+	p = skip(p);
+	if (!*(tty.ty_type = p))
+	    tty.ty_type = NULL;
+	else
+	    p = skip(p);
     }
     tty.ty_status = 0;
     tty.ty_window = NULL;
@@ -148,57 +173,56 @@ struct ttyent * getttyent(void)
 #define	scmp(e)	!strncmp(p, e, sizeof(e) - 1) && isspace(p[sizeof(e) - 1])
 #define	vcmp(e)	!strncmp(p, e, sizeof(e) - 1) && p[sizeof(e) - 1] == '='
     for (; *p; p = skip(p)) {
-		if (scmp(_TTYS_OFF))
-			tty.ty_status &= ~TTY_ON;
-		else if (scmp(_TTYS_ON))
-			tty.ty_status |= TTY_ON;
-		else if (scmp(_TTYS_SECURE))
-			tty.ty_status |= TTY_SECURE;
-		else if (vcmp(_TTYS_WINDOW))
-			tty.ty_window = value(p);
-		else
-			break;
+	if (scmp(_TTYS_OFF))
+	    tty.ty_status &= ~TTY_ON;
+	else if (scmp(_TTYS_ON))
+	    tty.ty_status |= TTY_ON;
+	else if (scmp(_TTYS_SECURE))
+	    tty.ty_status |= TTY_SECURE;
+	else if (vcmp(_TTYS_WINDOW))
+	    tty.ty_window = value(p);
+	else
+	    break;
     }
 
     if (zapchar == '#' || *p == '#')
-		while ((c = *++p) == ' ' || c == '\t')
-			;
+	while ((c = *++p) == ' ' || c == '\t')
+	    ;
     tty.ty_comment = p;
     if (*p == 0)
-		tty.ty_comment = 0;
-    if ((p = index(p, '\n')))
-		*p = '\0';
+	tty.ty_comment = 0;
+    if ((p = strchr(p, '\n')))
+	*p = '\0';
     retval = &tty;
 
  DONE:
     __STDIO_ALWAYS_THREADUNLOCK(tf);
     return retval;
 }
+libc_hidden_def(getttyent)
 
-int setttyent(void)
-{
-
-    if (tf) {
-		rewind(tf);
-		return (1);
-    } else if ((tf = fopen(_PATH_TTYS, "r"))) {
-		/* We do the locking ourselves.  */
-#ifdef __UCLIBC_HAS_THREADS__
-		__fsetlocking (tf, FSETLOCKING_BYCALLER);
-#endif
-		return (1);
-    }
-    return (0);
-}
-
+libc_hidden_proto(endttyent)
 int endttyent(void)
 {
     int rval;
 
     if (tf) {
-		rval = !(fclose(tf) == EOF);
-		tf = NULL;
-		return (rval);
+	rval = !(fclose(tf) == EOF);
+	tf = NULL;
+	return (rval);
     }
     return (1);
 }
+libc_hidden_def(endttyent)
+
+struct ttyent * getttynam(const char *_tty)
+{
+    register struct ttyent *t;
+
+    setttyent();
+    while ((t = getttyent()))
+	if (!strcmp(_tty, t->ty_name))
+	    break;
+    endttyent();
+    return (t);
+}
