@@ -98,8 +98,6 @@
  * Manuel
  */
 
-#define _GNU_SOURCE
-#define _ISOC99_SOURCE
 #include <errno.h>
 #include <stddef.h>
 #include <limits.h>
@@ -167,25 +165,18 @@
 /* Implementation-specific work functions. */
 
 extern size_t _wchar_utf8sntowcs(wchar_t *__restrict pwc, size_t wn,
-								 const char **__restrict src, size_t n,
-								 mbstate_t *ps, int allow_continuation);
+					const char **__restrict src, size_t n,
+					mbstate_t *ps, int allow_continuation) attribute_hidden;
 
 extern size_t _wchar_wcsntoutf8s(char *__restrict s, size_t n,
-								 const wchar_t **__restrict src, size_t wn);
-
-/* glibc extensions. */
-
-extern size_t __mbsnrtowcs(wchar_t *__restrict dst,
-						   const char **__restrict src,
-						   size_t NMC, size_t len, mbstate_t *__restrict ps);
-
-extern size_t __wcsnrtombs(char *__restrict dst,
-						   const wchar_t **__restrict src,
-						   size_t NWC, size_t len, mbstate_t *__restrict ps);
+					const wchar_t **__restrict src, size_t wn) attribute_hidden;
 
 /**********************************************************************/
 #ifdef L_btowc
 
+libc_hidden_proto(mbrtowc)
+
+libc_hidden_proto(btowc)
 wint_t btowc(int c)
 {
 #ifdef __CTYPE_HAS_8_BIT_LOCALES
@@ -216,6 +207,7 @@ wint_t btowc(int c)
 
 #endif /*  __CTYPE_HAS_8_BIT_LOCALES */
 }
+libc_hidden_def(btowc)
 
 #endif
 /**********************************************************************/
@@ -223,6 +215,8 @@ wint_t btowc(int c)
 
 /* Note: We completely ignore ps in all currently supported conversions. */
 
+libc_hidden_proto(wcrtomb)
+
 int wctob(wint_t c)
 {
 #ifdef __CTYPE_HAS_8_BIT_LOCALES
@@ -240,7 +234,7 @@ int wctob(wint_t c)
 
 	/* If we don't have 8-bit locale support, then this is trivial since
 	 * anything outside of 0-0x7f is illegal in C/POSIX and UTF-8 locales. */
-	
+
 	/* TODO: need unsigned version of wint_t... */
 /*  	return (((unsigned int)c) < 0x80) ? c : WEOF; */
 	return ((c >= 0) && (c < 0x80)) ? c : EOF;
@@ -252,29 +246,35 @@ int wctob(wint_t c)
 /**********************************************************************/
 #ifdef L_mbsinit
 
+libc_hidden_proto(mbsinit)
 int mbsinit(const mbstate_t *ps)
 {
 	return !ps || !ps->__mask;
 }
+libc_hidden_def(mbsinit)
 
 #endif
 /**********************************************************************/
 #ifdef L_mbrlen
 
-size_t mbrlen(const char *__restrict s, size_t n, mbstate_t *__restrict ps)
-	 __attribute__ ((__weak__, __alias__("__mbrlen")));
+libc_hidden_proto(mbrtowc)
 
-size_t __mbrlen(const char *__restrict s, size_t n, mbstate_t *__restrict ps)
+libc_hidden_proto(mbrlen)
+size_t mbrlen(const char *__restrict s, size_t n, mbstate_t *__restrict ps)
 {
 	static mbstate_t mbstate;	/* Rely on bss 0-init. */
 
 	return mbrtowc(NULL, s, n, (ps != NULL) ? ps : &mbstate);
 }
+libc_hidden_def(mbrlen)
 
 #endif
 /**********************************************************************/
 #ifdef L_mbrtowc
 
+libc_hidden_proto(mbsnrtowcs)
+
+libc_hidden_proto(mbrtowc)
 size_t mbrtowc(wchar_t *__restrict pwc, const char *__restrict s,
 			   size_t n, mbstate_t *__restrict ps)
 {
@@ -293,10 +293,17 @@ size_t mbrtowc(wchar_t *__restrict pwc, const char *__restrict s,
 		empty_string[0] = 0;	/* Init the empty string when necessary. */
 		s = empty_string;
 		n = 1;
+	} else if (*s == '\0') {
+    /* According to the ISO C 89 standard this is the expected behaviour.  */
+		return 0;
 	} else if (!n) {
 		/* TODO: change error code? */
+#if 0
 		return (ps->__mask && (ps->__wc == 0xffffU))
 			? ((size_t) -1) : ((size_t) -2);
+#else
+		return 0;
+#endif
 	}
 
 	p = s;
@@ -316,7 +323,7 @@ size_t mbrtowc(wchar_t *__restrict pwc, const char *__restrict s,
 #warning TODO: This adds a trailing nul!
 #endif /* __UCLIBC_MJN3_ONLY__ */
 
-	r = __mbsnrtowcs(wcbuf, &p, SIZE_MAX, 1, ps);
+	r = mbsnrtowcs(wcbuf, &p, SIZE_MAX, 1, ps);
 
 	if (((ssize_t) r) >= 0) {
 		if (pwc) {
@@ -325,14 +332,18 @@ size_t mbrtowc(wchar_t *__restrict pwc, const char *__restrict s,
 	}
 	return (size_t) r;
 }
+libc_hidden_def(mbrtowc)
 
 #endif
 /**********************************************************************/
 #ifdef L_wcrtomb
 
+libc_hidden_proto(wcsnrtombs)
+
 /* Note: We completely ignore ps in all currently supported conversions. */
 /* TODO: Check for valid state anyway? */
 
+libc_hidden_proto(wcrtomb)
 size_t wcrtomb(register char *__restrict s, wchar_t wc,
 			   mbstate_t *__restrict ps)
 {
@@ -352,22 +363,27 @@ size_t wcrtomb(register char *__restrict s, wchar_t wc,
 	pwc = wcbuf;
 	wcbuf[0] = wc;
 
-	r = __wcsnrtombs(s, &pwc, 1, MB_LEN_MAX, ps);
+	r = wcsnrtombs(s, &pwc, 1, MB_LEN_MAX, ps);
 	return (r != 0) ? r : 1;
 }
+libc_hidden_def(wcrtomb)
 
 #endif
 /**********************************************************************/
 #ifdef L_mbsrtowcs
 
+libc_hidden_proto(mbsnrtowcs)
+
+libc_hidden_proto(mbsrtowcs)
 size_t mbsrtowcs(wchar_t *__restrict dst, const char **__restrict src,
 				 size_t len, mbstate_t *__restrict ps)
 {
 	static mbstate_t mbstate;	/* Rely on bss 0-init. */
 
-	return __mbsnrtowcs(dst, src, SIZE_MAX, len,
+	return mbsnrtowcs(dst, src, SIZE_MAX, len,
 						((ps != NULL) ? ps : &mbstate));
 }
+libc_hidden_def(mbsrtowcs)
 
 #endif
 /**********************************************************************/
@@ -377,11 +393,15 @@ size_t mbsrtowcs(wchar_t *__restrict dst, const char **__restrict src,
 
  * TODO: Check for valid state anyway? */
 
+libc_hidden_proto(wcsnrtombs)
+
+libc_hidden_proto(wcsrtombs)
 size_t wcsrtombs(char *__restrict dst, const wchar_t **__restrict src,
 				 size_t len, mbstate_t *__restrict ps)
 {
-	return __wcsnrtombs(dst, src, SIZE_MAX, len, ps);
+	return wcsnrtombs(dst, src, SIZE_MAX, len, ps);
 }
+libc_hidden_def(wcsrtombs)
 
 #endif
 /**********************************************************************/
@@ -398,7 +418,7 @@ size_t wcsrtombs(char *__restrict dst, const wchar_t **__restrict src,
 #endif
 #endif
 
-size_t _wchar_utf8sntowcs(wchar_t *__restrict pwc, size_t wn,
+size_t attribute_hidden _wchar_utf8sntowcs(wchar_t *__restrict pwc, size_t wn,
 						  const char **__restrict src, size_t n,
 						  mbstate_t *ps, int allow_continuation)
 {
@@ -468,7 +488,8 @@ size_t _wchar_utf8sntowcs(wchar_t *__restrict pwc, size_t wn,
 #ifdef __UCLIBC_MJN3_ONLY__
 #warning TODO: Fix range for 16 bit wchar_t case.
 #endif
-			if ( ((unsigned char)(s[-1] - 0xc0)) < (0xfe - 0xc0) ) {
+			if (( ((unsigned char)(s[-1] - 0xc0)) < (0xfe - 0xc0) ) &&
+			(((unsigned char)s[-1] != 0xc0 ) && ((unsigned char)s[-1] != 0xc1 ))) {
 				goto START;
 			}
 		BAD:
@@ -569,7 +590,7 @@ size_t _wchar_utf8sntowcs(wchar_t *__restrict pwc, size_t wn,
 /**********************************************************************/
 #ifdef L__wchar_wcsntoutf8s
 
-size_t _wchar_wcsntoutf8s(char *__restrict s, size_t n,
+size_t attribute_hidden _wchar_wcsntoutf8s(char *__restrict s, size_t n,
 						  const wchar_t **__restrict src, size_t wn)
 {
 	register char *p;
@@ -675,15 +696,12 @@ size_t _wchar_wcsntoutf8s(char *__restrict s, size_t n,
 
 #endif
 /**********************************************************************/
-#ifdef L___mbsnrtowcs
+#ifdef L_mbsnrtowcs
 
 /* WARNING: We treat len as SIZE_MAX when dst is NULL! */
 
+libc_hidden_proto(mbsnrtowcs)
 size_t mbsnrtowcs(wchar_t *__restrict dst, const char **__restrict src,
-				  size_t NMC, size_t len, mbstate_t *__restrict ps)
-	 __attribute__ ((__weak__, __alias__("__mbsnrtowcs")));
-
-size_t __mbsnrtowcs(wchar_t *__restrict dst, const char **__restrict src,
 					size_t NMC, size_t len, mbstate_t *__restrict ps)
 {
 	static mbstate_t mbstate;	/* Rely on bss 0-init. */
@@ -781,21 +799,19 @@ size_t __mbsnrtowcs(wchar_t *__restrict dst, const char **__restrict src,
 	}
 	return len - count;
 }
+libc_hidden_def(mbsnrtowcs)
 
 #endif
 /**********************************************************************/
-#ifdef L___wcsnrtombs
+#ifdef L_wcsnrtombs
 
 /* WARNING: We treat len as SIZE_MAX when dst is NULL! */
 
 /* Note: We completely ignore ps in all currently supported conversions.
  * TODO: Check for valid state anyway? */
 
+libc_hidden_proto(wcsnrtombs)
 size_t wcsnrtombs(char *__restrict dst, const wchar_t **__restrict src,
-				  size_t NWC, size_t len, mbstate_t *__restrict ps)
-	 __attribute__ ((__weak__, __alias__("__wcsnrtombs")));
-
-size_t __wcsnrtombs(char *__restrict dst, const wchar_t **__restrict src,
 					size_t NWC, size_t len, mbstate_t *__restrict ps)
 {
 	const __uwchar_t *s;
@@ -857,7 +873,6 @@ size_t __wcsnrtombs(char *__restrict dst, const wchar_t **__restrict src,
 									+ (wc & ((1 << Cwc2c_TT_SHIFT)-1))];
 				}
 
-#define __WCHAR_REPLACEMENT_CHAR '?'
 #ifdef __WCHAR_REPLACEMENT_CHAR
 				*dst = (unsigned char) ( u ? u : __WCHAR_REPLACEMENT_CHAR );
 #else  /* __WCHAR_REPLACEMENT_CHAR */
@@ -903,11 +918,14 @@ size_t __wcsnrtombs(char *__restrict dst, const wchar_t **__restrict src,
 	}
 	return len - count;
 }
+libc_hidden_def(wcsnrtombs)
 
 #endif
 /**********************************************************************/
 #ifdef L_wcswidth
 
+libc_hidden_proto(wcswidth)
+
 #ifdef __UCLIBC_MJN3_ONLY__
 #warning REMINDER: If we start doing translit, wcwidth and wcswidth will need updating.
 #warning TODO: Update wcwidth to match latest by Kuhn.
@@ -917,111 +935,113 @@ size_t __wcsnrtombs(char *__restrict dst, const wchar_t **__restrict src,
 ( defined(__CTYPE_HAS_8_BIT_LOCALES) || defined(__CTYPE_HAS_UTF_8_LOCALES) )
 
 static const unsigned char new_idx[] = {
-	0,    5,    5,    6,   10,   15,   28,   39, 
-	48,   48,   71,   94,  113,  128,  139,  154, 
-	175,  186,  188,  188,  188,  188,  188,  188, 
-	203,  208,  208,  208,  208,  208,  208,  208, 
-	208,  219,  219,  219,  222,  222,  222,  222, 
-	222,  222,  222,  222,  222,  222,  222,  224, 
-	224,  231,  231,  231,  231,  231,  231,  231, 
-	231,  231,  231,  231,  231,  231,  231,  231, 
-	231,  231,  231,  231,  231,  231,  231,  231, 
-	231,  231,  231,  231,  231,  231,  231,  231, 
-	231,  231,  231,  231,  231,  231,  231,  231, 
-	231,  231,  231,  231,  231,  231,  231,  231, 
-	231,  231,  231,  231,  231,  231,  231,  231, 
-	231,  231,  231,  231,  231,  231,  231,  231, 
-	231,  231,  231,  231,  231,  231,  231,  231, 
-	231,  231,  231,  231,  231,  231,  231,  231, 
-	231,  231,  231,  231,  231,  231,  231,  231, 
-	231,  231,  231,  231,  231,  231,  231,  231, 
-	231,  231,  231,  231,  231,  231,  231,  231, 
-	231,  231,  231,  231,  231,  231,  231,  231, 
-	231,  231,  231,  231,  231,  233,  233,  233, 
-	233,  233,  233,  233,  234,  234,  234,  234, 
-	234,  234,  234,  234,  234,  234,  234,  234, 
-	234,  234,  234,  234,  234,  234,  234,  234, 
-	234,  234,  234,  234,  234,  234,  234,  234, 
-	234,  234,  234,  234,  234,  234,  234,  234, 
-	234,  234,  234,  234,  234,  234,  234,  234, 
-	236,  236,  236,  236,  236,  236,  236,  236, 
-	236,  236,  236,  236,  236,  236,  236,  236, 
-	236,  236,  236,  236,  236,  236,  236,  236, 
-	236,  236,  236,  236,  236,  236,  236,  236, 
-	236,  237,  237,  238,  241,  241,  242,  249, 
-	255, 
+	0,    5,    5,    6,   10,   15,   28,   39,
+	48,   48,   71,   94,  113,  128,  139,  154,
+	175,  186,  188,  188,  188,  188,  188,  188,
+	203,  208,  208,  208,  208,  208,  208,  208,
+	208,  219,  219,  219,  222,  222,  222,  222,
+	222,  222,  222,  222,  222,  222,  222,  224,
+	224,  231,  231,  231,  231,  231,  231,  231,
+	231,  231,  231,  231,  231,  231,  231,  231,
+	231,  231,  231,  231,  231,  231,  231,  231,
+	231,  231,  231,  231,  231,  231,  231,  231,
+	231,  231,  231,  231,  231,  231,  231,  231,
+	231,  231,  231,  231,  231,  231,  231,  231,
+	231,  231,  231,  231,  231,  231,  231,  231,
+	231,  231,  231,  231,  231,  231,  231,  231,
+	231,  231,  231,  231,  231,  231,  231,  231,
+	231,  231,  231,  231,  231,  231,  231,  231,
+	231,  231,  231,  231,  231,  231,  231,  231,
+	231,  231,  231,  231,  231,  231,  231,  231,
+	231,  231,  231,  231,  231,  231,  231,  231,
+	231,  231,  231,  231,  231,  231,  231,  231,
+	231,  231,  231,  231,  231,  233,  233,  233,
+	233,  233,  233,  233,  234,  234,  234,  234,
+	234,  234,  234,  234,  234,  234,  234,  234,
+	234,  234,  234,  234,  234,  234,  234,  234,
+	234,  234,  234,  234,  234,  234,  234,  234,
+	234,  234,  234,  234,  234,  234,  234,  234,
+	234,  234,  234,  234,  234,  234,  234,  234,
+	236,  236,  236,  236,  236,  236,  236,  236,
+	236,  236,  236,  236,  236,  236,  236,  236,
+	236,  236,  236,  236,  236,  236,  236,  236,
+	236,  236,  236,  236,  236,  236,  236,  236,
+	236,  237,  237,  238,  241,  241,  242,  249,
+	255,
 };
 
 static const unsigned char new_tbl[] = {
-	0x00, 0x01, 0x20, 0x7f, 0xa0, 0x00, 0x00, 0x50, 
-	0x60, 0x70, 0x00, 0x83, 0x87, 0x88, 0x8a, 0x00, 
-	0x91, 0xa2, 0xa3, 0xba, 0xbb, 0xbe, 0xbf, 0xc0, 
-	0xc1, 0xc3, 0xc4, 0xc5, 0x00, 0x4b, 0x56, 0x70, 
-	0x71, 0xd6, 0xe5, 0xe7, 0xe9, 0xea, 0xee, 0x00, 
-	0x0f, 0x10, 0x11, 0x12, 0x30, 0x4b, 0xa6, 0xb1, 
-	0x00, 0x01, 0x03, 0x3c, 0x3d, 0x41, 0x49, 0x4d, 
-	0x4e, 0x51, 0x55, 0x62, 0x64, 0x81, 0x82, 0xbc, 
-	0xbd, 0xc1, 0xc5, 0xcd, 0xce, 0xe2, 0xe4, 0x00, 
-	0x02, 0x03, 0x3c, 0x3d, 0x41, 0x43, 0x47, 0x49, 
-	0x4b, 0x4e, 0x70, 0x72, 0x81, 0x83, 0xbc, 0xbd, 
-	0xc1, 0xc6, 0xc7, 0xc9, 0xcd, 0xce, 0x00, 0x01, 
-	0x02, 0x3c, 0x3d, 0x3f, 0x40, 0x41, 0x44, 0x4d, 
-	0x4e, 0x56, 0x57, 0x82, 0x83, 0xc0, 0xc1, 0xcd, 
-	0xce, 0x00, 0x3e, 0x41, 0x46, 0x49, 0x4a, 0x4e, 
-	0x55, 0x57, 0xbf, 0xc0, 0xc6, 0xc7, 0xcc, 0xce, 
-	0x00, 0x41, 0x44, 0x4d, 0x4e, 0xca, 0xcb, 0xd2, 
-	0xd5, 0xd6, 0xd7, 0x00, 0x31, 0x32, 0x34, 0x3b, 
-	0x47, 0x4f, 0xb1, 0xb2, 0xb4, 0xba, 0xbb, 0xbd, 
-	0xc8, 0xce, 0x00, 0x18, 0x1a, 0x35, 0x36, 0x37, 
-	0x38, 0x39, 0x3a, 0x71, 0x7f, 0x80, 0x85, 0x86, 
-	0x88, 0x90, 0x98, 0x99, 0xbd, 0xc6, 0xc7, 0x00, 
-	0x2d, 0x31, 0x32, 0x33, 0x36, 0x38, 0x39, 0x3a, 
-	0x58, 0x5a, 0x00, 0x60, 0x00, 0x12, 0x15, 0x32, 
-	0x35, 0x52, 0x54, 0x72, 0x74, 0xb7, 0xbe, 0xc6, 
-	0xc7, 0xc9, 0xd4, 0x00, 0x0b, 0x0f, 0xa9, 0xaa, 
-	0x00, 0x0b, 0x10, 0x2a, 0x2f, 0x60, 0x64, 0x6a, 
-	0x70, 0xd0, 0xeb, 0x00, 0x29, 0x2b, 0x00, 0x80, 
-	0x00, 0x2a, 0x30, 0x3f, 0x40, 0x99, 0x9b, 0x00, 
-	0xd0, 0x00, 0x00, 0xa4, 0x00, 0x00, 0x00, 0x1e, 
-	0x1f, 0x00, 0x00, 0x10, 0x20, 0x24, 0x30, 0x70, 
-	0xff, 0x00, 0x61, 0xe0, 0xe7, 0xf9, 0xfc, 
+	0x00, 0x01, 0x20, 0x7f, 0xa0, 0x00, 0x00, 0x50,
+	0x60, 0x70, 0x00, 0x83, 0x87, 0x88, 0x8a, 0x00,
+	0x91, 0xa2, 0xa3, 0xba, 0xbb, 0xbe, 0xbf, 0xc0,
+	0xc1, 0xc3, 0xc4, 0xc5, 0x00, 0x4b, 0x56, 0x70,
+	0x71, 0xd6, 0xe5, 0xe7, 0xe9, 0xea, 0xee, 0x00,
+	0x0f, 0x10, 0x11, 0x12, 0x30, 0x4b, 0xa6, 0xb1,
+	0x00, 0x01, 0x03, 0x3c, 0x3d, 0x41, 0x49, 0x4d,
+	0x4e, 0x51, 0x55, 0x62, 0x64, 0x81, 0x82, 0xbc,
+	0xbd, 0xc1, 0xc5, 0xcd, 0xce, 0xe2, 0xe4, 0x00,
+	0x02, 0x03, 0x3c, 0x3d, 0x41, 0x43, 0x47, 0x49,
+	0x4b, 0x4e, 0x70, 0x72, 0x81, 0x83, 0xbc, 0xbd,
+	0xc1, 0xc6, 0xc7, 0xc9, 0xcd, 0xce, 0x00, 0x01,
+	0x02, 0x3c, 0x3d, 0x3f, 0x40, 0x41, 0x44, 0x4d,
+	0x4e, 0x56, 0x57, 0x82, 0x83, 0xc0, 0xc1, 0xcd,
+	0xce, 0x00, 0x3e, 0x41, 0x46, 0x49, 0x4a, 0x4e,
+	0x55, 0x57, 0xbf, 0xc0, 0xc6, 0xc7, 0xcc, 0xce,
+	0x00, 0x41, 0x44, 0x4d, 0x4e, 0xca, 0xcb, 0xd2,
+	0xd5, 0xd6, 0xd7, 0x00, 0x31, 0x32, 0x34, 0x3b,
+	0x47, 0x4f, 0xb1, 0xb2, 0xb4, 0xba, 0xbb, 0xbd,
+	0xc8, 0xce, 0x00, 0x18, 0x1a, 0x35, 0x36, 0x37,
+	0x38, 0x39, 0x3a, 0x71, 0x7f, 0x80, 0x85, 0x86,
+	0x88, 0x90, 0x98, 0x99, 0xbd, 0xc6, 0xc7, 0x00,
+	0x2d, 0x31, 0x32, 0x33, 0x36, 0x38, 0x39, 0x3a,
+	0x58, 0x5a, 0x00, 0x60, 0x00, 0x12, 0x15, 0x32,
+	0x35, 0x52, 0x54, 0x72, 0x74, 0xb7, 0xbe, 0xc6,
+	0xc7, 0xc9, 0xd4, 0x00, 0x0b, 0x0f, 0xa9, 0xaa,
+	0x00, 0x0b, 0x10, 0x2a, 0x2f, 0x60, 0x64, 0x6a,
+	0x70, 0xd0, 0xeb, 0x00, 0x29, 0x2b, 0x00, 0x80,
+	0x00, 0x2a, 0x30, 0x3f, 0x40, 0x99, 0x9b, 0x00,
+	0xd0, 0x00, 0x00, 0xa4, 0x00, 0x00, 0x00, 0x1e,
+	0x1f, 0x00, 0x00, 0x10, 0x20, 0x24, 0x30, 0x70,
+	0xff, 0x00, 0x61, 0xe0, 0xe7, 0xf9, 0xfc,
 };
 
 static const signed char new_wtbl[] = {
-	0,   -1,    1,   -1,    1,    1,    0,    1, 
-	0,    1,    1,    0,    1,    0,    1,    1, 
-	0,    1,    0,    1,    0,    1,    0,    1, 
-	0,    1,    0,    1,    1,    0,    1,    0, 
-	1,    0,    1,    0,    1,    0,    1,    1, 
-	0,    1,    0,    1,    0,    1,    0,    1, 
-	1,    0,    1,    0,    1,    0,    1,    0, 
-	1,    0,    1,    0,    1,    0,    1,    0, 
-	1,    0,    1,    0,    1,    0,    1,    1, 
-	0,    1,    0,    1,    0,    1,    0,    1, 
-	0,    1,    0,    1,    0,    1,    0,    1, 
-	0,    1,    0,    1,    0,    1,    1,    0, 
-	1,    0,    1,    0,    1,    0,    1,    0, 
-	1,    0,    1,    0,    1,    0,    1,    0, 
-	1,    1,    0,    1,    0,    1,    0,    1, 
-	0,    1,    0,    1,    0,    1,    0,    1, 
-	1,    0,    1,    0,    1,    0,    1,    0, 
-	1,    0,    1,    1,    0,    1,    0,    1, 
-	0,    1,    0,    1,    0,    1,    0,    1, 
-	0,    1,    1,    0,    1,    0,    1,    0, 
-	1,    0,    1,    0,    1,    0,    1,    0, 
-	1,    0,    1,    0,    1,    0,    1,    1, 
-	0,    1,    0,    1,    0,    1,    0,    1, 
-	0,    1,    2,    0,    1,    0,    1,    0, 
-	1,    0,    1,    0,    1,    0,    1,    0, 
-	1,    0,    1,    1,    0,    1,    0,    1, 
-	1,    0,    1,    0,    1,    0,    1,    0, 
-	1,    0,    1,    1,    2,    1,    1,    2, 
-	2,    0,    2,    1,    2,    0,    2,    2, 
-	1,    1,    2,    1,    1,    2,    1,    0, 
-	1,    1,    0,    1,    0,    1,    2,    1, 
-	0,    2,    1,    2,    1,    0,    1, 
+	0,   -1,    1,   -1,    1,    1,    0,    1,
+	0,    1,    1,    0,    1,    0,    1,    1,
+	0,    1,    0,    1,    0,    1,    0,    1,
+	0,    1,    0,    1,    1,    0,    1,    0,
+	1,    0,    1,    0,    1,    0,    1,    1,
+	0,    1,    0,    1,    0,    1,    0,    1,
+	1,    0,    1,    0,    1,    0,    1,    0,
+	1,    0,    1,    0,    1,    0,    1,    0,
+	1,    0,    1,    0,    1,    0,    1,    1,
+	0,    1,    0,    1,    0,    1,    0,    1,
+	0,    1,    0,    1,    0,    1,    0,    1,
+	0,    1,    0,    1,    0,    1,    1,    0,
+	1,    0,    1,    0,    1,    0,    1,    0,
+	1,    0,    1,    0,    1,    0,    1,    0,
+	1,    1,    0,    1,    0,    1,    0,    1,
+	0,    1,    0,    1,    0,    1,    0,    1,
+	1,    0,    1,    0,    1,    0,    1,    0,
+	1,    0,    1,    1,    0,    1,    0,    1,
+	0,    1,    0,    1,    0,    1,    0,    1,
+	0,    1,    1,    0,    1,    0,    1,    0,
+	1,    0,    1,    0,    1,    0,    1,    0,
+	1,    0,    1,    0,    1,    0,    1,    1,
+	0,    1,    0,    1,    0,    1,    0,    1,
+	0,    1,    2,    0,    1,    0,    1,    0,
+	1,    0,    1,    0,    1,    0,    1,    0,
+	1,    0,    1,    1,    0,    1,    0,    1,
+	1,    0,    1,    0,    1,    0,    1,    0,
+	1,    0,    1,    1,    2,    1,    1,    2,
+	2,    0,    2,    1,    2,    0,    2,    2,
+	1,    1,    2,    1,    1,    2,    1,    0,
+	1,    1,    0,    1,    0,    1,    2,    1,
+	0,    2,    1,    2,    1,    0,    1,
 };
 
+libc_hidden_proto(wcsnrtombs)
+
 int wcswidth(const wchar_t *pwcs, size_t n)
 {
     int h, l, m, count;
@@ -1030,9 +1050,9 @@ int wcswidth(const wchar_t *pwcs, size_t n)
 
 	if (ENCODING == __ctype_encoding_7_bit) {
 		size_t i;
-		
+
 		for (i = 0 ; (i < n) && pwcs[i] ; i++) {
-			if (pwcs[i] != ((unsigned char)(pwcs[i]))) {
+			if (pwcs[i] != (pwcs[i] & 0x7f)) {
 				return -1;
 			}
 		}
@@ -1042,7 +1062,7 @@ int wcswidth(const wchar_t *pwcs, size_t n)
 		mbstate_t mbstate;
 
 		mbstate.__mask = 0;			/* Initialize the mbstate. */
-		if (__wcsnrtombs(NULL, &pwcs, n, SIZE_MAX, &mbstate) == ((size_t) - 1)) {
+		if (wcsnrtombs(NULL, &pwcs, n, SIZE_MAX, &mbstate) == ((size_t) - 1)) {
 			return -1;
 		}
 	}
@@ -1051,7 +1071,7 @@ int wcswidth(const wchar_t *pwcs, size_t n)
 	/* For stricter handling of allowed unicode values... see comments above. */
 	else if (ENCODING == __ctype_encoding_utf8) {
 		size_t i;
-		
+
 		for (i = 0 ; (i < n) && pwcs[i] ; i++) {
 			if ( (((__uwchar_t)((pwcs[i]) - 0xfffeU)) < 2)
 				 || (((__uwchar_t)((pwcs[i]) - 0xd800U)) < (0xe000U - 0xd800U))
@@ -1123,6 +1143,13 @@ int wcswidth(const wchar_t *pwcs, size_t n)
 {
 	int count;
 	wchar_t wc;
+	size_t i;
+
+	for (i = 0 ; (i < n) && pwcs[i] ; i++) {
+		if (pwcs[i] != (pwcs[i] & 0x7f)) {
+			return -1;
+		}
+	}
 
     for (count = 0 ; n && (wc = *pwcs++) ; n--) {
 		if (wc <= 0xff) {
@@ -1142,10 +1169,14 @@ int wcswidth(const wchar_t *pwcs, size_t n)
 
 #endif /*  __UCLIBC_HAS_LOCALE__ */
 
+libc_hidden_def(wcswidth)
+
 #endif
 /**********************************************************************/
 #ifdef L_wcwidth
 
+libc_hidden_proto(wcswidth)
+
 int wcwidth(wchar_t wc)
 {
     return wcswidth(&wc, 1);
@@ -1214,13 +1245,15 @@ enum {
  *
  */
 
+extern const unsigned char __iconv_codesets[];
+libc_hidden_proto(__iconv_codesets)
 const unsigned char __iconv_codesets[] =
 	"\x0a\xe0""WCHAR_T\x00"		/* superset of UCS-4 but platform-endian */
 #if __BYTE_ORDER == __BIG_ENDIAN
 	"\x08\xec""UCS-4\x00"		/* always BE */
 	"\x0a\xec""UCS-4BE\x00"
 	"\x0a\xed""UCS-4LE\x00"
-	"\x09\fe4""UTF-32\x00"		/* platform endian with BOM */
+	"\x09\xe4""UTF-32\x00"		/* platform endian with BOM */
 	"\x0b\xe4""UTF-32BE\x00"
 	"\x0b\xe5""UTF-32LE\x00"
 	"\x08\xe2""UCS-2\x00"		/* always BE */
@@ -1246,6 +1279,9 @@ const unsigned char __iconv_codesets[] =
 	"\x08\x02""UTF-8\x00"
 	"\x0b\x01""US-ASCII\x00"
 	"\x07\x01""ASCII";			/* Must be last! (special case to save a nul) */
+libc_hidden_data_def(__iconv_codesets)
+
+/* Experimentally off - libc_hidden_proto(strcasecmp) */
 
 static int find_codeset(const char *name)
 {
@@ -1479,7 +1515,7 @@ size_t weak_function iconv(iconv_t cd, char **__restrict inbuf,
 					}
 					wc += (wc2 << 16);
 				} else if (px->tocodeset & 1) wc = bswap_16(wc);
-			}				
+			}
 			(*outbuf)[0] = (char)((unsigned char)(wc));
 			(*outbuf)[1] = (char)((unsigned char)(wc >> 8));
 			if (inco == 4) {
@@ -1542,10 +1578,7 @@ size_t weak_function iconv(iconv_t cd, char **__restrict inbuf,
 /**********************************************************************/
 #ifdef L_iconv_main
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
-#include <wchar.h>
 #include <iconv.h>
 #include <stdarg.h>
 #include <libgen.h>
@@ -1558,10 +1591,10 @@ extern const unsigned char __iconv_codesets[];
 char *progname;
 int hide_errors;
 
-static void error_msg(const char *fmt, ...) 
+static void error_msg(const char *fmt, ...)
 	 __attribute__ ((noreturn, format (printf, 1, 2)));
 
-static void error_msg(const char *fmt, ...) 
+static void error_msg(const char *fmt, ...)
 {
 	va_list arg;
 
