@@ -33,8 +33,57 @@
  */
 
 #define _ISOC99_SOURCE			/* for ULLONG primarily... */
-#define _GNU_SOURCE
 #include <limits.h>
+#include <stdint.h>
+/* Work around gcc's refusal to create aliases.
+ * TODO: Add in a define to disable the aliases? */
+
+#if UINT_MAX == ULONG_MAX
+#ifdef L_labs
+#define abs __ignore_abs
+#endif
+#ifdef L_atol
+#define atoi __ignore_atoi
+#endif
+#endif
+#if defined(ULLONG_MAX) && (ULLONG_MAX == ULONG_MAX)
+#ifdef L_labs
+#define llabs __ignore_llabs
+#endif
+#ifdef L_atol
+#define atoll __ignore_atoll
+#endif
+#ifdef L_strtol
+#define strtoll __ignore_strtoll
+#endif
+#ifdef L_strtoul
+#define strtoull __ignore_strtoull
+#endif
+#ifdef L_wcstol
+#define wcstoll __ignore_wcstoll
+#endif
+#ifdef L_wcstoul
+#define wcstoull __ignore_wcstoull
+#endif
+#ifdef L_strtol_l
+#define strtoll_l __ignore_strtoll_l
+#endif
+#ifdef L_strtoul_l
+#define strtoull_l __ignore_strtoull_l
+#endif
+#ifdef L_wcstol_l
+#define wcstoll_l __ignore_wcstoll_l
+#endif
+#ifdef L_wcstoul_l
+#define wcstoull_l __ignore_wcstoull_l
+#endif
+#endif
+#if defined(ULLONG_MAX) && (ULLONG_MAX == UINTMAX_MAX)
+#if defined L_labs || defined L_llabs
+#define imaxabs __ignore_imaxabs
+#endif
+#endif
+
 #include <stdint.h>
 #include <inttypes.h>
 #include <ctype.h>
@@ -42,26 +91,6 @@
 #include <assert.h>
 #include <unistd.h>
 
-/* Work around gcc's refusal to create aliases. 
- * TODO: Add in a define to disable the aliases? */
-
-#if UINT_MAX == ULONG_MAX
-#define atoi __ignore_atoi
-#define abs __ignore_abs
-#endif
-#if defined(ULLONG_MAX) && (ULLONG_MAX == ULONG_MAX)
-#define llabs __ignore_llabs
-#define atoll __ignore_atoll
-#define strtoll __ignore_strtoll
-#define strtoull __ignore_strtoull
-#define wcstoll __ignore_wcstoll
-#define wcstoull __ignore_wcstoull
-#define strtoll_l __ignore_strtoll_l
-#define strtoull_l __ignore_strtoull_l
-#define wcstoll_l __ignore_wcstoll_l
-#define wcstoull_l __ignore_wcstoull_l
-#endif
-
 #include <stdlib.h>
 #include <locale.h>
 
@@ -111,49 +140,32 @@
 
 #endif /* __UCLIBC_HAS_LOCALE__ */
 
-#if UINT_MAX == ULONG_MAX
-#undef atoi
-#undef abs
-#endif
-#if defined(ULLONG_MAX) && (ULLONG_MAX == ULONG_MAX)
-#undef llabs
-#undef atoll
-#undef strtoll
-#undef strtoull
-#undef wcstoll
-#undef wcstoull
-#undef strtoll_l
-#undef strtoull_l
-#undef wcstoll_l
-#undef wcstoull_l
-#endif /* __UCLIBC_HAS_WCHAR__ */
-
 /**********************************************************************/
 #ifdef __UCLIBC_HAS_XLOCALE__
 
 extern unsigned long
 _stdlib_strto_l_l(register const char * __restrict str,
 				  char ** __restrict endptr, int base, int sflag,
-				  __locale_t locale_arg);
+				  __locale_t locale_arg) attribute_hidden;
 
 #if defined(ULLONG_MAX)
 extern unsigned long long
 _stdlib_strto_ll_l(register const char * __restrict str,
 				   char ** __restrict endptr, int base, int sflag,
-				  __locale_t locale_arg);
+				  __locale_t locale_arg) attribute_hidden;
 #endif
 
 #ifdef __UCLIBC_HAS_WCHAR__
 extern unsigned long
 _stdlib_wcsto_l_l(register const wchar_t * __restrict str,
 				  wchar_t ** __restrict endptr, int base, int sflag,
-				  __locale_t locale_arg);
+				  __locale_t locale_arg) attribute_hidden;
 
 #if defined(ULLONG_MAX)
 extern unsigned long long
 _stdlib_wcsto_ll_l(register const wchar_t * __restrict str,
 				   wchar_t ** __restrict endptr, int base, int sflag,
-				  __locale_t locale_arg);
+				  __locale_t locale_arg) attribute_hidden;
 #endif
 #endif /* __UCLIBC_HAS_WCHAR__ */
 
@@ -163,28 +175,30 @@ _stdlib_wcsto_ll_l(register const wchar_t * __restrict str,
 
 extern unsigned long
 _stdlib_strto_l(register const char * __restrict str,
-				char ** __restrict endptr, int base, int sflag);
+				char ** __restrict endptr, int base, int sflag) attribute_hidden;
 
 #if defined(ULLONG_MAX)
 extern unsigned long long
 _stdlib_strto_ll(register const char * __restrict str,
-				 char ** __restrict endptr, int base, int sflag);
+				 char ** __restrict endptr, int base, int sflag) attribute_hidden;
 #endif
 
 #ifdef __UCLIBC_HAS_WCHAR__
 extern unsigned long
 _stdlib_wcsto_l(register const wchar_t * __restrict str,
-				wchar_t ** __restrict endptr, int base, int sflag);
+				wchar_t ** __restrict endptr, int base, int sflag) attribute_hidden;
 
 #if defined(ULLONG_MAX)
 extern unsigned long long
 _stdlib_wcsto_ll(register const wchar_t * __restrict str,
-				 wchar_t ** __restrict endptr, int base, int sflag);
+				 wchar_t ** __restrict endptr, int base, int sflag) attribute_hidden;
 #endif
 #endif /* __UCLIBC_HAS_WCHAR__ */
 /**********************************************************************/
 #ifdef L_atof
 
+libc_hidden_proto(strtod)
+
 double atof(const char *nptr)
 {
 	return strtod(nptr, (char **) NULL);
@@ -207,50 +221,62 @@ int abs(int j)
 /**********************************************************************/
 #ifdef L_labs
 
-#if UINT_MAX == ULONG_MAX
-strong_alias(labs,abs)
-#endif
-
-#if defined(ULLONG_MAX) && (ULLONG_MAX == ULONG_MAX)
-strong_alias(labs,llabs)
-#endif
-
-#if ULONG_MAX == UINTMAX_MAX
-strong_alias(labs,imaxabs)
-#endif
-
 long int labs(long int j)
 {
 	return (j >= 0) ? j : -j;
 }
 
+#if UINT_MAX == ULONG_MAX
+#undef abs
+extern __typeof(labs) abs;
+strong_alias(labs,abs)
+#endif
+
+#if defined(ULLONG_MAX) && (ULLONG_MAX == ULONG_MAX)
+#undef llabs
+extern __typeof(labs) llabs;
+strong_alias(labs,llabs)
+#endif
+
+#if ULONG_MAX == UINTMAX_MAX
+#undef imaxabs
+extern __typeof(labs) imaxabs;
+strong_alias(labs,imaxabs)
+#endif
+
 #endif
 /**********************************************************************/
 #ifdef L_llabs
 
 #if defined(ULLONG_MAX) && (LLONG_MAX > LONG_MAX)
 
-#if (ULLONG_MAX == UINTMAX_MAX)
-strong_alias(llabs,imaxabs)
-#endif
-
 long long int llabs(long long int j)
 {
 	return (j >= 0) ? j : -j;
 }
 
+#if (ULLONG_MAX == UINTMAX_MAX)
+#undef imaxabs
+extern __typeof(llabs) imaxabs;
+strong_alias(llabs,imaxabs)
+#endif
+
 #endif /* defined(ULLONG_MAX) && (LLONG_MAX > LONG_MAX) */
 
 #endif
 /**********************************************************************/
 #ifdef L_atoi
 
-#if INT_MAX < LONG_MAX 
+#if INT_MAX < LONG_MAX
 
+libc_hidden_proto(strtol)
+
+libc_hidden_proto(atoi)
 int atoi(const char *nptr)
 {
 	return (int) strtol(nptr, (char **) NULL, 10);
 }
+libc_hidden_def(atoi)
 
 #endif /* INT_MAX < LONG_MAX  */
 
@@ -258,18 +284,28 @@ int atoi(const char *nptr)
 /**********************************************************************/
 #ifdef L_atol
 
-#if UINT_MAX == ULONG_MAX
-strong_alias(atol,atoi)
-#endif
-
-#if defined(ULLONG_MAX) && (ULLONG_MAX == ULONG_MAX)
-strong_alias(atol,atoll)
-#endif
+libc_hidden_proto(strtol)
 
+libc_hidden_proto(atol)
 long atol(const char *nptr)
 {
 	return strtol(nptr, (char **) NULL, 10);
 }
+libc_hidden_def(atol)
+
+#if UINT_MAX == ULONG_MAX
+#undef atoi
+extern __typeof(atol) atoi;
+libc_hidden_proto(atoi)
+strong_alias(atol,atoi)
+libc_hidden_def(atoi)
+#endif
+
+#if defined(ULLONG_MAX) && (ULLONG_MAX == ULONG_MAX)
+#undef atoll
+extern __typeof(atol) atoll;
+strong_alias(atol,atoll)
+#endif
 
 #endif
 /**********************************************************************/
@@ -277,6 +313,8 @@ long atol(const char *nptr)
 
 #if defined(ULLONG_MAX) && (LLONG_MAX > LONG_MAX)
 
+libc_hidden_proto(strtoll)
+
 long long atoll(const char *nptr)
 {
 	return strtoll(nptr, (char **) NULL, 10);
@@ -288,32 +326,45 @@ long long atoll(const char *nptr)
 /**********************************************************************/
 #if defined(L_strtol) || defined(L_strtol_l)
 
+libc_hidden_proto(__XL_NPP(strtol))
+long __XL_NPP(strtol)(const char * __restrict str, char ** __restrict endptr,
+				  int base __LOCALE_PARAM)
+{
+	return __XL_NPP(_stdlib_strto_l)(str, endptr, base, 1 __LOCALE_ARG);
+}
+libc_hidden_def(__XL_NPP(strtol))
+
 #if (ULONG_MAX == UINTMAX_MAX) && !defined(L_strtol_l)
 strong_alias(strtol,strtoimax)
 #endif
 
 #if defined(ULLONG_MAX) && (ULLONG_MAX == ULONG_MAX)
-strong_alias(__XL(strtol),__XL(strtoll))
+#ifdef L_strtol_l
+#undef strtoll_l
+#else
+#undef strtoll
+#endif
+extern __typeof(__XL_NPP(strtol)) __XL_NPP(strtoll);
+libc_hidden_proto(__XL_NPP(strtoll))
+strong_alias(__XL_NPP(strtol),__XL_NPP(strtoll))
+libc_hidden_def(__XL_NPP(strtoll))
 #endif
 
-long __XL(strtol)(const char * __restrict str, char ** __restrict endptr,
-				  int base   __LOCALE_PARAM )
-{
-    return __XL_NPP(_stdlib_strto_l)(str, endptr, base, 1   __LOCALE_ARG );
-}
-
-__XL_ALIAS(strtol)
-
-#if defined(__sparc__) // to be able to use libstdc++ ontop 
-weak_alias (__XL(strtol),__strtol_internal)
-#endif
-     
 #endif
 /**********************************************************************/
 #if defined(L_strtoll) || defined(L_strtoll_l)
 
 #if defined(ULLONG_MAX) && (LLONG_MAX > LONG_MAX)
 
+libc_hidden_proto(__XL_NPP(strtoll))
+long long __XL_NPP(strtoll)(const char * __restrict str,
+						char ** __restrict endptr, int base
+						__LOCALE_PARAM)
+{
+	return (long long) __XL_NPP(_stdlib_strto_ll)(str, endptr, base, 1 __LOCALE_ARG);
+}
+libc_hidden_def(__XL_NPP(strtoll))
+
 #if !defined(L_strtoll_l)
 #if (ULLONG_MAX == UINTMAX_MAX)
 strong_alias(strtoll,strtoimax)
@@ -321,53 +372,53 @@ strong_alias(strtoll,strtoimax)
 strong_alias(strtoll,strtoq)
 #endif
 
-long long __XL(strtoll)(const char * __restrict str,
-						char ** __restrict endptr, int base
-						__LOCALE_PARAM )
-{
-    return (long long) __XL_NPP(_stdlib_strto_ll)(str, endptr, base, 1
-												  __LOCALE_ARG );
-}
-
-__XL_ALIAS(strtoll)
-
-#if defined(__sparc__) // to be able to use libstdc++ ontop 
-weak_alias (__XL(strtoll),__strtoll_internal)
-#endif
-     
 #endif /* defined(ULLONG_MAX) && (LLONG_MAX > LONG_MAX) */
 
 #endif
 /**********************************************************************/
 #if defined(L_strtoul) || defined(L_strtoul_l)
 
+libc_hidden_proto(__XL_NPP(strtoul))
+unsigned long __XL_NPP(strtoul)(const char * __restrict str,
+							char ** __restrict endptr, int base
+							__LOCALE_PARAM)
+{
+	return __XL_NPP(_stdlib_strto_l)(str, endptr, base, 0 __LOCALE_ARG);
+}
+libc_hidden_def(__XL_NPP(strtoul))
+
 #if (ULONG_MAX == UINTMAX_MAX) && !defined(L_strtoul_l)
 strong_alias(strtoul,strtoumax)
 #endif
 
 #if defined(ULLONG_MAX) && (ULLONG_MAX == ULONG_MAX)
-strong_alias(__XL(strtoul),__XL(strtoull))
+#ifdef L_strtoul_l
+#undef strtoull_l
+#else
+#undef strtoull
+#endif
+extern __typeof(__XL_NPP(strtoul)) __XL_NPP(strtoull);
+libc_hidden_proto(__XL_NPP(strtoull))
+strong_alias(__XL_NPP(strtoul),__XL_NPP(strtoull))
+libc_hidden_def(__XL_NPP(strtoull))
 #endif
 
-unsigned long __XL(strtoul)(const char * __restrict str,
-							char ** __restrict endptr, int base
-							__LOCALE_PARAM )
-{
-    return __XL_NPP(_stdlib_strto_l)(str, endptr, base, 0   __LOCALE_ARG );
-}
 
-__XL_ALIAS(strtoul)
-
-#if defined(__sparc__) // to be able to use libstdc++ ontop 
-weak_alias (__XL(strtoul),__strtoul_internal)
-#endif
-     
 #endif
 /**********************************************************************/
 #if defined(L_strtoull) || defined(L_strtoull_l)
 
 #if defined(ULLONG_MAX) && (LLONG_MAX > LONG_MAX)
 
+libc_hidden_proto(__XL_NPP(strtoull))
+unsigned long long __XL_NPP(strtoull)(const char * __restrict str,
+								  char ** __restrict endptr, int base
+								  __LOCALE_PARAM)
+{
+	return __XL_NPP(_stdlib_strto_ll)(str, endptr, base, 0 __LOCALE_ARG);
+}
+libc_hidden_def(__XL_NPP(strtoull))
+
 #if !defined(L_strtoull_l)
 #if (ULLONG_MAX == UINTMAX_MAX)
 strong_alias(strtoull,strtoumax)
@@ -375,19 +426,6 @@ strong_alias(strtoull,strtoumax)
 strong_alias(strtoull,strtouq)
 #endif
 
-unsigned long long __XL(strtoull)(const char * __restrict str,
-								  char ** __restrict endptr, int base
-								  __LOCALE_PARAM )
-{
-    return __XL_NPP(_stdlib_strto_ll)(str, endptr, base, 0   __LOCALE_ARG );
-}
-
-__XL_ALIAS(strtoull)
-
-#if defined(__sparc__) // to be able to use libstdc++ ontop 
-weak_alias (__XL(strtoull),__strtoull_internal)
-#endif
-     
 #endif /* defined(ULLONG_MAX) && (LLONG_MAX > LONG_MAX) */
 
 #endif
@@ -412,8 +450,10 @@ weak_alias (__XL(strtoull),__strtoull_internal)
 
 /**********************************************************************/
 #if defined(L__stdlib_wcsto_l) || defined(L__stdlib_wcsto_l_l)
+#ifndef L__stdlib_strto_l
 #define L__stdlib_strto_l
 #endif
+#endif
 
 #if defined(L__stdlib_strto_l) || defined(L__stdlib_strto_l_l)
 
@@ -425,8 +465,10 @@ weak_alias (__XL(strtoull),__strtoull_internal)
 #define Wuchar __uwchar_t
 #ifdef __UCLIBC_DO_XLOCALE
 #define ISSPACE(C) iswspace_l((C), locale_arg)
+libc_hidden_proto(iswspace_l)
 #else
 #define ISSPACE(C) iswspace((C))
+libc_hidden_proto(iswspace)
 #endif
 
 #else  /* defined(L__stdlib_wcsto_l) || defined(L__stdlib_wcsto_l_l) */
@@ -435,15 +477,17 @@ weak_alias (__XL(strtoull),__strtoull_internal)
 #define Wuchar unsigned char
 #ifdef __UCLIBC_DO_XLOCALE
 #define ISSPACE(C) isspace_l((C), locale_arg)
+libc_hidden_proto(isspace_l)
 #else
 #define ISSPACE(C) isspace((C))
+libc_hidden_proto(isspace)
 #endif
 
 #endif /* defined(L__stdlib_wcsto_l) || defined(L__stdlib_wcsto_l_l) */
 
 #if defined(__UCLIBC_HAS_XLOCALE__) && !defined(__UCLIBC_DO_XLOCALE)
 
-unsigned long _stdlib_strto_l(register const Wchar * __restrict str,
+unsigned long attribute_hidden _stdlib_strto_l(register const Wchar * __restrict str,
 							  Wchar ** __restrict endptr, int base,
 							  int sflag)
 {
@@ -456,35 +500,41 @@ unsigned long _stdlib_strto_l(register const Wchar * __restrict str,
 /* This is the main work fuction which handles both strtol (sflag = 1) and
  * strtoul (sflag = 0). */
 
-unsigned long __XL_NPP(_stdlib_strto_l)(register const Wchar * __restrict str,
-										Wchar ** __restrict endptr, int base,
-										int sflag   __LOCALE_PARAM )
-{
-    unsigned long number, cutoff;
-#if _STRTO_ENDPTR
-    const Wchar *fail_char;
-#define SET_FAIL(X)       fail_char = (X)
-#else
-#define SET_FAIL(X)       ((void)(X)) /* Keep side effects. */
+#ifdef __UCLIBC_HAS_XLOCALE__
+libc_hidden_proto(__ctype_b_loc)
+#elif defined __UCLIBC_HAS_CTYPE_TABLES__
+libc_hidden_proto(__ctype_b)
 #endif
-    unsigned char negative, digit, cutoff_digit;
+
+unsigned long attribute_hidden __XL_NPP(_stdlib_strto_l)(register const Wchar * __restrict str,
+										Wchar ** __restrict endptr, int base,
+										int sflag __LOCALE_PARAM)
+{
+	unsigned long number, cutoff;
+#if _STRTO_ENDPTR
+	const Wchar *fail_char;
+#define SET_FAIL(X) fail_char = (X)
+#else
+#define SET_FAIL(X) ((void)(X)) /* Keep side effects. */
+#endif
+	unsigned char negative, digit, cutoff_digit;
 
 	assert(((unsigned int)sflag) <= 1);
 
 	SET_FAIL(str);
 
-    while (ISSPACE(*str)) { /* Skip leading whitespace. */
+	while (ISSPACE(*str)) { /* Skip leading whitespace. */
 		++str;
-    }
+	}
 
-    /* Handle optional sign. */
-    negative = 0;
-    switch(*str) {
+	/* Handle optional sign. */
+	negative = 0;
+	switch (*str) {
 		case '-': negative = 1;	/* Fall through to increment str. */
 		case '+': ++str;
-    }
+	}
 
-    if (!(base & ~0x10)) {		/* Either dynamic (base = 0) or base 16. */
+	if (!(base & ~0x10)) {		/* Either dynamic (base = 0) or base 16. */
 		base += 10;				/* Default is 10 (26). */
 		if (*str == '0') {
 			SET_FAIL(++str);
@@ -498,19 +548,19 @@ unsigned long __XL_NPP(_stdlib_strto_l)(register const Wchar * __restrict str,
 		if (base > 16) {		/* Adjust in case base wasn't dynamic. */
 			base = 16;
 		}
-    }
+	}
 
 	number = 0;
 
-    if (((unsigned)(base - 2)) < 35) { /* Legal base. */
+	if (((unsigned)(base - 2)) < 35) { /* Legal base. */
 		cutoff_digit = ULONG_MAX % base;
 		cutoff = ULONG_MAX / base;
 		do {
-			digit = (((Wuchar)(*str - '0')) <= 9)
-				? (*str - '0')
-				: ((*str >= 'A')
-				   ? (((0x20|(*str)) - 'a' + 10)) /* WARNING: assumes ascii. */
-					  : 40);
+			digit = ((Wuchar)(*str - '0') <= 9)
+				? /* 0..9 */ (*str - '0')
+				: /* else */ (((Wuchar)(0x20 | *str) >= 'a') /* WARNING: assumes ascii. */
+				   ? /* >= A/a */ ((Wuchar)(0x20 | *str) - ('a' - 10))
+				   : /* else   */ 40 /* bad value */);
 
 			if (digit >= base) {
 				break;
@@ -530,13 +580,13 @@ unsigned long __XL_NPP(_stdlib_strto_l)(register const Wchar * __restrict str,
 	}
 
 #if _STRTO_ENDPTR
-    if (endptr) {
+	if (endptr) {
 		*endptr = (Wchar *) fail_char;
-    }
+	}
 #endif
 
 	{
-		unsigned long tmp = ((negative)
+		unsigned long tmp = (negative
 							 ? ((unsigned long)(-(1+LONG_MIN)))+1
 							 : LONG_MAX);
 		if (sflag && (number > tmp)) {
@@ -554,8 +604,10 @@ unsigned long __XL_NPP(_stdlib_strto_l)(register const Wchar * __restrict str,
 #endif
 /**********************************************************************/
 #if defined(L__stdlib_wcsto_ll) || defined(L__stdlib_wcsto_ll_l)
+#ifndef L__stdlib_strto_ll
 #define L__stdlib_strto_ll
 #endif
+#endif
 
 #if defined(L__stdlib_strto_ll) || defined(L__stdlib_strto_ll_l)
 
@@ -568,8 +620,10 @@ unsigned long __XL_NPP(_stdlib_strto_l)(register const Wchar * __restrict str,
 #define Wuchar __uwchar_t
 #ifdef __UCLIBC_DO_XLOCALE
 #define ISSPACE(C) iswspace_l((C), locale_arg)
+libc_hidden_proto(iswspace_l)
 #else
 #define ISSPACE(C) iswspace((C))
+libc_hidden_proto(iswspace)
 #endif
 
 #else  /* defined(L__stdlib_wcsto_ll) || defined(L__stdlib_wcsto_ll_l) */
@@ -578,15 +632,17 @@ unsigned long __XL_NPP(_stdlib_strto_l)(register const Wchar * __restrict str,
 #define Wuchar unsigned char
 #ifdef __UCLIBC_DO_XLOCALE
 #define ISSPACE(C) isspace_l((C), locale_arg)
+libc_hidden_proto(isspace_l)
 #else
 #define ISSPACE(C) isspace((C))
+libc_hidden_proto(isspace)
 #endif
 
 #endif /* defined(L__stdlib_wcsto_ll) || defined(L__stdlib_wcsto_ll_l) */
 
 #if defined(__UCLIBC_HAS_XLOCALE__) && !defined(__UCLIBC_DO_XLOCALE)
 
-unsigned long long _stdlib_strto_ll(register const Wchar * __restrict str,
+unsigned long long attribute_hidden _stdlib_strto_ll(register const Wchar * __restrict str,
 									Wchar ** __restrict endptr, int base,
 									int sflag)
 {
@@ -596,39 +652,42 @@ unsigned long long _stdlib_strto_ll(register const Wchar * __restrict str,
 
 #else  /* defined(__UCLIBC_HAS_XLOCALE__) && !defined(__UCLIBC_DO_XLOCALE) */
 
+#if !defined __UCLIBC_HAS_XLOCALE__ && defined __UCLIBC_HAS_CTYPE_TABLES__
+libc_hidden_proto(__ctype_b)
+#endif
 /* This is the main work fuction which handles both strtoll (sflag = 1) and
  * strtoull (sflag = 0). */
 
-unsigned long long __XL_NPP(_stdlib_strto_ll)(register const Wchar * __restrict str,
+unsigned long long attribute_hidden __XL_NPP(_stdlib_strto_ll)(register const Wchar * __restrict str,
 											  Wchar ** __restrict endptr, int base,
-											  int sflag   __LOCALE_PARAM )
+											  int sflag __LOCALE_PARAM)
 {
-    unsigned long long number;
+	unsigned long long number;
 #if _STRTO_ENDPTR
-    const Wchar *fail_char;
-#define SET_FAIL(X)       fail_char = (X)
+	const Wchar *fail_char;
+#define SET_FAIL(X) fail_char = (X)
 #else
-#define SET_FAIL(X)       ((void)(X)) /* Keep side effects. */
+#define SET_FAIL(X) ((void)(X)) /* Keep side effects. */
 #endif
 	unsigned int n1;
-    unsigned char negative, digit;
+	unsigned char negative, digit;
 
 	assert(((unsigned int)sflag) <= 1);
 
 	SET_FAIL(str);
 
-    while (ISSPACE(*str)) {		/* Skip leading whitespace. */
+	while (ISSPACE(*str)) {		/* Skip leading whitespace. */
 		++str;
-    }
+	}
 
-    /* Handle optional sign. */
-    negative = 0;
-    switch(*str) {
+	/* Handle optional sign. */
+	negative = 0;
+	switch (*str) {
 		case '-': negative = 1;	/* Fall through to increment str. */
 		case '+': ++str;
-    }
+	}
 
-    if (!(base & ~0x10)) {		/* Either dynamic (base = 0) or base 16. */
+	if (!(base & ~0x10)) {		/* Either dynamic (base = 0) or base 16. */
 		base += 10;				/* Default is 10 (26). */
 		if (*str == '0') {
 			SET_FAIL(++str);
@@ -642,17 +701,17 @@ unsigned long long __XL_NPP(_stdlib_strto_ll)(register const Wchar * __restrict
 		if (base > 16) {		/* Adjust in case base wasn't dynamic. */
 			base = 16;
 		}
-    }
+	}
 
 	number = 0;
 
-    if (((unsigned)(base - 2)) < 35) { /* Legal base. */
+	if (((unsigned)(base - 2)) < 35) { /* Legal base. */
 		do {
-			digit = (((Wuchar)(*str - '0')) <= 9)
-				? (*str - '0')
-				: ((*str >= 'A')
-				   ? (((0x20|(*str)) - 'a' + 10)) /* WARNING: assumes ascii. */
-					  : 40);
+			digit = ((Wuchar)(*str - '0') <= 9)
+				? /* 0..9 */ (*str - '0')
+				: /* else */ (((Wuchar)(0x20 | *str) >= 'a') /* WARNING: assumes ascii. */
+				   ? /* >= A/a */ ((Wuchar)(0x20 | *str) - ('a' - 10))
+				   : /* else   */ 40 /* bad value */);
 
 			if (digit >= base) {
 				break;
@@ -683,9 +742,9 @@ unsigned long long __XL_NPP(_stdlib_strto_ll)(register const Wchar * __restrict
 	}
 
 #if _STRTO_ENDPTR
-    if (endptr) {
+	if (endptr) {
 		*endptr = (Wchar *) fail_char;
-    }
+	}
 #endif
 
 	{
@@ -709,7 +768,7 @@ unsigned long long __XL_NPP(_stdlib_strto_ll)(register const Wchar * __restrict
 /**********************************************************************/
 /* Made _Exit() an alias for _exit(), as per C99. */
 /*  #ifdef L__Exit */
-
+/*  libc_hidden_proto(_exit) */
 /*  void _Exit(int status) */
 /*  { */
 /*  	_exit(status); */
@@ -757,16 +816,17 @@ void *bsearch(const void *key, const void *base, size_t /* nmemb */ high,
  * calculation, as well as to reduce the generated code size with
  * bcc and gcc. */
 
-void qsort (void  *base,
-            size_t nel,
-            size_t width,
-            int (*comp)(const void *, const void *))
+libc_hidden_proto(qsort)
+void qsort(void  *base,
+           size_t nel,
+           size_t width,
+           int (*comp)(const void *, const void *))
 {
 	size_t wgap, i, j, k;
 	char tmp;
 
 	if ((nel > 1) && (width > 0)) {
-		assert( nel <= ((size_t)(-1)) / width ); /* check for overflow */
+		assert(nel <= ((size_t)(-1)) / width); /* check for overflow */
 		wgap = 0;
 		do {
 			wgap = 3 * wgap + 1;
@@ -786,7 +846,7 @@ void qsort (void  *base,
 					j -= wgap;
 					a = j + ((char *)base);
 					b = a + wgap;
-					if ( (*comp)(a, b) <= 0 ) {
+					if ((*comp)(a, b) <= 0) {
 						break;
 					}
 					k = width;
@@ -794,7 +854,7 @@ void qsort (void  *base,
 						tmp = *a;
 						*a++ = *b;
 						*b++ = tmp;
-					} while ( --k );
+					} while (--k);
 				} while (j >= wgap);
 				i += width;
 			} while (i < nel);
@@ -802,6 +862,7 @@ void qsort (void  *base,
 		} while (wgap);
 	}
 }
+libc_hidden_def(qsort)
 
 /* ---------- original snippets version below ---------- */
 
@@ -814,40 +875,36 @@ void qsort (void  *base,
 
 #include <stddef.h>
 
-void ssort (void  *base,
-            size_t nel,
-            size_t width,
-            int (*comp)(const void *, const void *))
+void ssort(void  *base,
+           size_t nel,
+           size_t width,
+           int (*comp)(const void *, const void *))
 {
-      size_t wnel, gap, wgap, i, j, k;
-      char *a, *b, tmp;
+	size_t wnel, gap, wgap, i, j, k;
+	char *a, *b, tmp;
 
-      wnel = width * nel;
-      for (gap = 0; ++gap < nel;)
-            gap *= 3;
-      while ( gap /= 3 )
-      {
-            wgap = width * gap;
-            for (i = wgap; i < wnel; i += width)
-            {
-                  for (j = i - wgap; ;j -= wgap)
-                  {
-                        a = j + (char *)base;
-                        b = a + wgap;
-                        if ( (*comp)(a, b) <= 0 )
-                              break;
-                        k = width;
-                        do
-                        {
-                              tmp = *a;
-                              *a++ = *b;
-                              *b++ = tmp;
-                        } while ( --k );
-                        if (j < wgap)
-                              break;
-                  }
-            }
-      }
+	wnel = width * nel;
+	for (gap = 0; ++gap < nel;)
+		gap *= 3;
+	while ((gap /= 3) != 0) {
+		wgap = width * gap;
+		for (i = wgap; i < wnel; i += width) {
+			for (j = i - wgap; ;j -= wgap) {
+				a = j + (char *)base;
+				b = a + wgap;
+				if ((*comp)(a, b) <= 0)
+					break;
+				k = width;
+				do {
+					tmp = *a;
+					*a++ = *b;
+					*b++ = tmp;
+				} while (--k);
+				if (j < wgap)
+					break;
+			}
+		}
+	}
 }
 #endif
 
@@ -855,6 +912,7 @@ void ssort (void  *base,
 /**********************************************************************/
 #ifdef L__stdlib_mb_cur_max
 
+libc_hidden_proto(_stdlib_mb_cur_max)
 size_t _stdlib_mb_cur_max(void)
 {
 #ifdef __CTYPE_HAS_UTF_8_LOCALES
@@ -868,11 +926,38 @@ size_t _stdlib_mb_cur_max(void)
 	return 1;
 #endif
 }
+libc_hidden_def(_stdlib_mb_cur_max)
 
 #endif
+
+#ifdef __UCLIBC_HAS_LOCALE__
+/*
+ * The following function return 1 if the encoding is stateful, 0 if stateless.
+ * To note, until now all the supported encoding are stateless.
+ */
+
+static __always_inline int is_stateful(unsigned char encoding)
+{
+	switch (encoding)
+	{
+		case __ctype_encoding_7_bit:
+		case __ctype_encoding_utf8:
+		case __ctype_encoding_8_bit:
+			return 0;
+		default:
+			assert(0);
+			return -1;
+	}
+}
+#else
+#define is_stateful(encoding) 0
+#endif
+
 /**********************************************************************/
 #ifdef L_mblen
 
+libc_hidden_proto(mbrlen)
+
 int mblen(register const char *s, size_t n)
 {
 	static mbstate_t state;
@@ -880,17 +965,21 @@ int mblen(register const char *s, size_t n)
 
 	if (!s) {
 		state.__mask = 0;
-#ifdef __CTYPE_HAS_UTF_8_LOCALES
-		return ENCODING == __ctype_encoding_utf8;
-#else
-		return 0;
-#endif
+		/*
+			In this case we have to return 0 because the only multibyte supported encoding
+			is utf-8, that is a stateless encoding. See mblen() documentation.
+		*/
+		return is_stateful(ENCODING);
 	}
 
+	if (*s == '\0')
+		/* According to the ISO C 89 standard this is the expected behaviour.  */
+		return 0;
+
 	if ((r = mbrlen(s, n, &state)) == (size_t) -2) {
 		/* TODO: Should we set an error state? */
 		state.__wc = 0xffffU;	/* Make sure we're in an error state. */
-		return (size_t) -1;		/* TODO: Change error code above? */
+		return -1;		/* TODO: Change error code above? */
 	}
 	return r;
 }
@@ -899,6 +988,8 @@ int mblen(register const char *s, size_t n)
 /**********************************************************************/
 #ifdef L_mbtowc
 
+libc_hidden_proto(mbrtowc)
+
 int mbtowc(wchar_t *__restrict pwc, register const char *__restrict s, size_t n)
 {
 	static mbstate_t state;
@@ -906,17 +997,22 @@ int mbtowc(wchar_t *__restrict pwc, register const char *__restrict s, size_t n)
 
 	if (!s) {
 		state.__mask = 0;
-#ifdef __CTYPE_HAS_UTF_8_LOCALES
-		return ENCODING == __ctype_encoding_utf8;
-#else
-		return 0;
-#endif
+		/*
+			In this case we have to return 0 because the only multibyte supported encoding
+			is utf-8, that is a stateless encoding. See mbtowc() documentation.
+		*/
+
+		return is_stateful(ENCODING);
 	}
 
+	if (*s == '\0')
+		/* According to the ISO C 89 standard this is the expected behaviour.  */
+		return 0;
+
 	if ((r = mbrtowc(pwc, s, n, &state)) == (size_t) -2) {
 		/* TODO: Should we set an error state? */
 		state.__wc = 0xffffU;	/* Make sure we're in an error state. */
-		return (size_t) -1;		/* TODO: Change error code above? */
+		return -1;		/* TODO: Change error code above? */
 	}
 	return r;
 }
@@ -927,15 +1023,18 @@ int mbtowc(wchar_t *__restrict pwc, register const char *__restrict s, size_t n)
 
 /* Note: We completely ignore state in all currently supported conversions. */
 
+libc_hidden_proto(wcrtomb)
+
 int wctomb(register char *__restrict s, wchar_t swc)
 {
 	return (!s)
 		?
-#ifdef __CTYPE_HAS_UTF_8_LOCALES
-		(ENCODING == __ctype_encoding_utf8)
-#else
-		0						/* Encoding is stateless. */
-#endif
+		/*
+			In this case we have to return 0 because the only multibyte supported encoding
+			is utf-8, that is a stateless encoding. See wctomb() documentation.
+		*/
+
+		is_stateful(ENCODING)
 		: ((ssize_t) wcrtomb(s, swc, NULL));
 }
 
@@ -943,6 +1042,8 @@ int wctomb(register char *__restrict s, wchar_t swc)
 /**********************************************************************/
 #ifdef L_mbstowcs
 
+libc_hidden_proto(mbsrtowcs)
+
 size_t mbstowcs(wchar_t * __restrict pwcs, const char * __restrict s, size_t n)
 {
 	mbstate_t state;
@@ -958,6 +1059,8 @@ size_t mbstowcs(wchar_t * __restrict pwcs, const char * __restrict s, size_t n)
 
 /* Note: We completely ignore state in all currently supported conversions. */
 
+libc_hidden_proto(wcsrtombs)
+
 size_t wcstombs(char * __restrict s, const wchar_t * __restrict pwcs, size_t n)
 {
 	const wchar_t *e = pwcs;	/* Needed because of restrict. */
@@ -969,21 +1072,29 @@ size_t wcstombs(char * __restrict s, const wchar_t * __restrict pwcs, size_t n)
 /**********************************************************************/
 #if defined(L_wcstol) || defined(L_wcstol_l)
 
+libc_hidden_proto(__XL_NPP(wcstol))
+long __XL_NPP(wcstol)(const wchar_t * __restrict str,
+				  wchar_t ** __restrict endptr, int base __LOCALE_PARAM)
+{
+	return __XL_NPP(_stdlib_wcsto_l)(str, endptr, base, 1 __LOCALE_ARG);
+}
+libc_hidden_def(__XL_NPP(wcstol))
+
 #if (ULONG_MAX == UINTMAX_MAX) && !defined(L_wcstol_l)
 strong_alias(wcstol,wcstoimax)
 #endif
 
 #if defined(ULLONG_MAX) && (ULLONG_MAX == ULONG_MAX)
-strong_alias(__XL(wcstol),__XL(wcstoll))
+#ifdef L_wcstol_l
+#undef wcstoll_l
+#else
+#undef wcstoll
+#endif
+extern __typeof(__XL_NPP(wcstol)) __XL_NPP(wcstoll);
+libc_hidden_proto(__XL_NPP(wcstoll))
+strong_alias(__XL_NPP(wcstol),__XL_NPP(wcstoll))
+libc_hidden_def(__XL_NPP(wcstoll))
 #endif
-
-long __XL(wcstol)(const wchar_t * __restrict str,
-				  wchar_t ** __restrict endptr, int base   __LOCALE_PARAM )
-{
-    return __XL_NPP(_stdlib_wcsto_l)(str, endptr, base, 1   __LOCALE_ARG );
-}
-
-__XL_ALIAS(wcstol)
 
 #endif
 /**********************************************************************/
@@ -991,6 +1102,15 @@ __XL_ALIAS(wcstol)
 
 #if defined(ULLONG_MAX) && (LLONG_MAX > LONG_MAX)
 
+libc_hidden_proto(__XL_NPP(wcstoll))
+long long __XL_NPP(wcstoll)(const wchar_t * __restrict str,
+						wchar_t ** __restrict endptr, int base
+						__LOCALE_PARAM)
+{
+	return (long long) __XL_NPP(_stdlib_wcsto_ll)(str, endptr, base, 1 __LOCALE_ARG);
+}
+libc_hidden_def(__XL_NPP(wcstoll))
+
 #if !defined(L_wcstoll_l)
 #if (ULLONG_MAX == UINTMAX_MAX)
 strong_alias(wcstoll,wcstoimax)
@@ -998,38 +1118,36 @@ strong_alias(wcstoll,wcstoimax)
 strong_alias(wcstoll,wcstoq)
 #endif
 
-long long __XL(wcstoll)(const wchar_t * __restrict str,
-						wchar_t ** __restrict endptr, int base
-						__LOCALE_PARAM )
-{
-    return (long long) __XL_NPP(_stdlib_wcsto_ll)(str, endptr, base, 1
-												  __LOCALE_ARG );
-}
-
-__XL_ALIAS(wcstoll)
-
 #endif /* defined(ULLONG_MAX) && (LLONG_MAX > LONG_MAX) */
 
 #endif
 /**********************************************************************/
 #if defined(L_wcstoul) || defined(L_wcstoul_l)
 
+libc_hidden_proto(__XL_NPP(wcstoul))
+unsigned long __XL_NPP(wcstoul)(const wchar_t * __restrict str,
+							wchar_t ** __restrict endptr, int base
+							__LOCALE_PARAM)
+{
+	return __XL_NPP(_stdlib_wcsto_l)(str, endptr, base, 0 __LOCALE_ARG);
+}
+libc_hidden_def(__XL_NPP(wcstoul))
+
 #if (ULONG_MAX == UINTMAX_MAX) && !defined(L_wcstoul_l)
 strong_alias(wcstoul,wcstoumax)
 #endif
 
 #if defined(ULLONG_MAX) && (ULLONG_MAX == ULONG_MAX)
-strong_alias(__XL(wcstoul),__XL(wcstoull))
+#ifdef L_wcstoul_l
+#undef wcstoull_l
+#else
+#undef wcstoull
+#endif
+extern __typeof(__XL_NPP(wcstoul)) __XL_NPP(wcstoull);
+libc_hidden_proto(__XL_NPP(wcstoull))
+strong_alias(__XL_NPP(wcstoul),__XL_NPP(wcstoull))
+libc_hidden_def(__XL_NPP(wcstoull))
 #endif
-
-unsigned long __XL(wcstoul)(const wchar_t * __restrict str,
-							wchar_t ** __restrict endptr, int base
-							__LOCALE_PARAM )
-{
-    return __XL_NPP(_stdlib_wcsto_l)(str, endptr, base, 0   __LOCALE_ARG );
-}
-
-__XL_ALIAS(wcstoul)
 
 #endif
 /**********************************************************************/
@@ -1037,6 +1155,15 @@ __XL_ALIAS(wcstoul)
 
 #if defined(ULLONG_MAX) && (LLONG_MAX > LONG_MAX)
 
+libc_hidden_proto(__XL_NPP(wcstoull))
+unsigned long long __XL_NPP(wcstoull)(const wchar_t * __restrict str,
+								  wchar_t ** __restrict endptr, int base
+								  __LOCALE_PARAM)
+{
+	return __XL_NPP(_stdlib_wcsto_ll)(str, endptr, base, 0 __LOCALE_ARG);
+}
+libc_hidden_def(__XL_NPP(wcstoull))
+
 #if !defined(L_wcstoull_l)
 #if (ULLONG_MAX == UINTMAX_MAX)
 strong_alias(wcstoull,wcstoumax)
@@ -1044,17 +1171,7 @@ strong_alias(wcstoull,wcstoumax)
 strong_alias(wcstoull,wcstouq)
 #endif
 
-unsigned long long __XL(wcstoull)(const wchar_t * __restrict str,
-								  wchar_t ** __restrict endptr, int base
-								  __LOCALE_PARAM )
-{
-    return __XL_NPP(_stdlib_wcsto_ll)(str, endptr, base, 0   __LOCALE_ARG );
-}
-
-__XL_ALIAS(wcstoull)
-
 #endif /* defined(ULLONG_MAX) && (LLONG_MAX > LONG_MAX) */
 
 #endif
 /**********************************************************************/
-
