@@ -16,31 +16,20 @@
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA.  */
 
-#include <features.h>
+#include <_lfs_64.h>
+
+#ifdef __UCLIBC_HAS_LFS__
 #include <fcntl.h>
 #include <sys/types.h>
 
-#if defined __UCLIBC_HAS_LFS__
-
-#if defined _FILE_OFFSET_BITS && _FILE_OFFSET_BITS != 64 
-#undef _FILE_OFFSET_BITS
-#define _FILE_OFFSET_BITS   64
-#endif
-#ifndef __USE_LARGEFILE64
-# define __USE_LARGEFILE64      1
-#endif
-/* We absolutely do _NOT_ want interfaces silently
- *  * renamed under us or very bad things will happen... */
-#ifdef __USE_FILE_OFFSET64
-# undef __USE_FILE_OFFSET64
-#endif
-
-#undef	creat
+extern __typeof(open64) __libc_open64;
+libc_hidden_proto(__libc_open64)
+extern __typeof(creat64) __libc_creat64;
 
 /* Create FILE with protections MODE.  */
-int creat64 (const char *file, mode_t mode)
+int __libc_creat64 (const char *file, mode_t mode)
 {
-    return open64 (file, O_WRONLY|O_CREAT|O_TRUNC, mode);
+    return __libc_open64 (file, O_WRONLY|O_CREAT|O_TRUNC, mode);
 }
+weak_alias(__libc_creat64,creat64)
 #endif /* __UCLIBC_HAS_LFS__ */
-
