@@ -2,22 +2,24 @@
 /*
  * getpriority() for uClibc
  *
- * Copyright (C) 2000-2004 by Erik Andersen <andersen@codepoet.org>
+ * Copyright (C) 2000-2006 Erik Andersen <andersen@uclibc.org>
  *
- * GNU Library General Public License (LGPL) version 2 or later.
+ * Licensed under the LGPL v2.1, see the file COPYING.LIB in this tarball.
  */
 
-#include "syscalls.h"
+#include <sys/syscall.h>
 #include <sys/resource.h>
 
+libc_hidden_proto(getpriority)
+
 #define __NR___syscall_getpriority __NR_getpriority
-static inline _syscall2(int, __syscall_getpriority,
-		__priority_which_t, which, id_t, who);
+static __inline__ _syscall2(int, __syscall_getpriority,
+		__priority_which_t, which, id_t, who)
 
 /* The return value of __syscall_getpriority is biased by this value
  * to avoid returning negative values.  */
 #define PZERO 20
-int getpriority(enum __priority_which which, id_t who)
+int getpriority(__priority_which_t which, id_t who)
 {
 	int res;
 
@@ -26,3 +28,4 @@ int getpriority(enum __priority_which which, id_t who)
 		res = PZERO - res;
 	return res;
 }
+libc_hidden_def(getpriority)
