@@ -21,8 +21,9 @@
 #include <unistd.h>
 #include <sys/syscall.h>
 
-void *__curbrk = 0;
+void *__curbrk attribute_hidden = 0;
 
+libc_hidden_proto(brk)
 int brk (void *addr)
 {
   void *newbrk;
@@ -30,7 +31,7 @@ int brk (void *addr)
   {
     register long int res __asm__ ("$2");
 
-    asm ("move\t$4,%2\n\t"
+    __asm__ ("move\t$4,%2\n\t"
 	 "li\t%0,%1\n\t"
 	 "syscall"		/* Perform the system call.  */
 	 : "=r" (res)
@@ -48,3 +49,4 @@ int brk (void *addr)
 
   return 0;
 }
+libc_hidden_def(brk)
