@@ -1,8 +1,12 @@
 #ifndef _BITS_STAT_STRUCT_H
 #define _BITS_STAT_STRUCT_H
 
-/* This file provides whatever this particular arch's kernel thinks 
- * struct kernel_stat should look like...  It turns out each arch has a 
+#ifndef _LIBC
+#error bits/kernel_stat.h is for internal uClibc use only!
+#endif
+
+/* This file provides whatever this particular arch's kernel thinks
+ * struct kernel_stat should look like...  It turns out each arch has a
  * different opinion on the subject... */
 #include <endian.h>
 
@@ -30,10 +34,10 @@ struct kernel_stat {
 };
 
 struct kernel_stat64 {
-#if defined(__BIG_ENDIAN__)
+#if (__BYTE_ORDER == __BIG_ENDIAN)
 	unsigned char   __pad0b[6];
 	unsigned short	st_dev;
-#elif defined(__LITTLE_ENDIAN__)
+#elif (__BYTE_ORDER == __LITTLE_ENDIAN)
 	unsigned short	st_dev;
 	unsigned char	__pad0b[6];
 #else
@@ -48,7 +52,7 @@ struct kernel_stat64 {
 	unsigned long	st_uid;
 	unsigned long	st_gid;
 
-#if defined(__BIG_ENDIAN__)
+#if (__BYTE_ORDER == __BIG_ENDIAN)
 	unsigned char	__pad3b[6];
 	unsigned short	st_rdev;
 #else /* Must be little */
@@ -60,7 +64,7 @@ struct kernel_stat64 {
 	long long	st_size;
 	unsigned long	st_blksize;
 
-#if defined(__BIG_ENDIAN__)
+#if (__BYTE_ORDER == __BIG_ENDIAN)
 	unsigned long	__pad4;		/* Future possible st_blocks hi bits */
 	unsigned long	st_blocks;	/* Number 512-byte blocks allocated. */
 #else /* Must be little */
