@@ -64,12 +64,12 @@
 #include <pwd.h>
 #include <string.h>
 #include <crypt.h>
+#include "libcrypt.h"
 
-/* Re-entrantify me -- all this junk needs to be in 
+/* Re-entrantify me -- all this junk needs to be in
  * struct crypt_data to make this really reentrant... */
 static u_char	inv_key_perm[64];
 static u_char	inv_comp_perm[56];
-static u_char	u_sbox[8][64];
 static u_char	un_pbox[32];
 static u_int32_t en_keysl[16], en_keysr[16];
 static u_int32_t de_keysl[16], de_keysr[16];
@@ -82,8 +82,8 @@ static u_int32_t old_salt;
 static u_int32_t old_rawkey0, old_rawkey1;
 
 
-/* Static stuff that stays resident and doesn't change after 
- * being initialized, and therefore doesn't need to be made 
+/* Static stuff that stays resident and doesn't change after
+ * being initialized, and therefore doesn't need to be made
  * reentrant. */
 static u_char	init_perm[64], final_perm[64];
 static u_char	m_sbox[4][4096];
@@ -193,10 +193,9 @@ static const u_int32_t bits32[32] =
 };
 
 static const u_char	bits8[8] = { 0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01 };
-static const u_int32_t *bits28, *bits24;
 
 
-static int 
+static int
 ascii_to_bin(char ch)
 {
 	if (ch > 'z')
@@ -217,12 +216,15 @@ ascii_to_bin(char ch)
 static void
 des_init(void)
 {
-	int	i, j, b, k, inbit, obit;
-	u_int32_t	*p, *il, *ir, *fl, *fr;
 	static int des_initialised = 0;
 
+	int	i, j, b, k, inbit, obit;
+	u_int32_t	*p, *il, *ir, *fl, *fr;
+	const u_int32_t *bits28, *bits24;
+	u_char	u_sbox[8][64];
+
 	if (des_initialised==1)
-	    return;
+		return;
 
 	old_rawkey0 = old_rawkey1 = 0L;
 	saltbits = 0L;
@@ -349,7 +351,7 @@ des_init(void)
 
 
 static void
-setup_salt(long salt)
+setup_salt(u_int32_t salt)
 {
 	u_int32_t	obit, saltbit;
 	int	i;
@@ -370,7 +372,7 @@ setup_salt(long salt)
 }
 
 
-static int
+static void
 des_setkey(const char *key)
 {
 	u_int32_t	k0, k1, rawkey0, rawkey1;
@@ -390,7 +392,7 @@ des_setkey(const char *key)
 		 * has bad parity anyway) in order to simplify the starting
 		 * conditions.
 		 */
-		return(0);
+		return;
 	}
 	old_rawkey0 = rawkey0;
 	old_rawkey1 = rawkey1;
@@ -446,7 +448,6 @@ des_setkey(const char *key)
 				| comp_maskr[6][(t1 >> 7) & 0x7f]
 				| comp_maskr[7][t1 & 0x7f];
 	}
-	return(0);
 }
 
 
@@ -611,7 +612,7 @@ setkey(const char *key)
 			if (*key++ & 1)
 				p[i] |= bits8[j];
 	}
-	des_setkey(p);
+	des_setkey((char *)p);
 }
 
 
@@ -625,7 +626,7 @@ encrypt(char *block, int flag)
 	des_init();
 
 	setup_salt(0L);
-	p = block;
+	p = (u_char*)block;
 	for (i = 0; i < 2; i++) {
 		io[i] = 0L;
 		for (j = 0; j < 32; j++)
@@ -638,8 +639,7 @@ encrypt(char *block, int flag)
 			block[(i << 5) | j] = (io[i] & bits32[j]) ? 1 : 0;
 }
 
-char *
-__des_crypt(const char *key, const char *setting)
+char *__des_crypt(const unsigned char *key, const unsigned char *setting)
 {
 	u_int32_t	count, salt, l, r0, r1, keybuf[2];
 	u_char		*p, *q;
@@ -657,8 +657,7 @@ __des_crypt(const char *key, const char *setting)
 		if (*(q - 1))
 			key++;
 	}
-	if (des_setkey((char *)keybuf))
-		return(NULL);
+	des_setkey((char *)keybuf);
 
 #if 0
 	if (*setting == _PASSWORD_EFMT1) {
@@ -687,8 +686,7 @@ __des_crypt(const char *key, const char *setting)
 			while (q - (u_char *)keybuf - 8 && *key)
 				*q++ ^= *key++ << 1;
 
-			if (des_setkey((char *)keybuf))
-				return(NULL);
+			des_setkey((char *)keybuf);
 		}
 		strncpy(output, setting, 9);
 
@@ -701,7 +699,7 @@ __des_crypt(const char *key, const char *setting)
 		 */
 		output[9] = '\0';
 		p = (u_char *)output + strlen(output);
-	} else 
+	} else
 #endif
 	{
 		/*
