@@ -44,12 +44,12 @@
 # include <byteswap.h>
 #else
 
-static inline uint16_t bswap_16(uint16_t x)
+static __inline__ uint16_t bswap_16(uint16_t x)
 {
 	return ((((x) & 0xff00) >> 8) | \
 	        (((x) & 0x00ff) << 8));
 }
-static inline uint32_t bswap_32(uint32_t x)
+static __inline__ uint32_t bswap_32(uint32_t x)
 {
 	return ((((x) & 0xff000000) >> 24) | \
 	        (((x) & 0x00ff0000) >>  8) | \
